﻿using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using QueryFeatures.QueryService;
using ThinkGeo.MapSuite.SilverlightEdition;
using System.IO;
using System.Collections.ObjectModel;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Windows.Input;

namespace QueryFeatures
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
        }

        private void Map1_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            ServerLayerOverlay overlay = new ServerLayerOverlay("NativeServer", "SilverlightMapConnector1");
            Map1.Overlays.Add(overlay);

            LayerOverlay shapeOverlay = new LayerOverlay();
            InMemoryFeatureLayer shapeLayer = new InMemoryFeatureLayer();
            shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Evergreen1;
            shapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            shapeOverlay.Layers.Add("ShapeLayer", shapeLayer);
            Map1.Overlays.Add("ShapeOverlay", shapeOverlay);

            Map1.CurrentExtent = new RectangleShape(-131.22, 55.05, -54.03, 16.91);
        }

        private void Map1_Click(object sender, MapClickEventArgs e)
        {
            QueryServiceSoapClient client = new QueryServiceSoapClient();
            client.GetFeatureInfoAsync(e.WorldPoint.X, e.WorldPoint.Y);
            client.GetFeatureInfoCompleted += new System.EventHandler<GetFeatureInfoCompletedEventArgs>(client_GetFeatureInfoCompleted);
            Cursor = Cursors.Wait;
        }

        void client_GetFeatureInfoCompleted(object sender, GetFeatureInfoCompletedEventArgs e)
        {
            byte[] buffer = Encoding.UTF8.GetBytes(e.Result);
            MemoryStream stream = new MemoryStream(buffer);
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(Collection<JsonFeature>));
            Collection<JsonFeature> jsonFeatures = serializer.ReadObject(stream) as Collection<JsonFeature>;

            LayerOverlay shapeOverlay = (LayerOverlay)Map1.Overlays["ShapeOverlay"];
            InMemoryFeatureLayer shapeLayer = (InMemoryFeatureLayer)shapeOverlay.Layers["ShapeLayer"];
            shapeLayer.InternalFeatures.Clear();

            foreach (JsonFeature jsonFeature in jsonFeatures)
            {
                shapeLayer.InternalFeatures.Add(jsonFeature.ToFeature());   
            }

            shapeOverlay.Refresh();
            Cursor = Cursors.Arrow;
        }
    }

    [DataContract]
    public class JsonFeature
    {
        private Dictionary<string, string> columnValues = new Dictionary<string, string>();

        [DataMember]
        public byte[] Wkb { get; set; }
        [DataMember]
        public Dictionary<string, string> ColumnValues { get { return columnValues; } set { columnValues = value; } }

        public Feature ToFeature()
        {
            Feature feature = new Feature(Wkb);
            foreach (string key in columnValues.Keys)
            {
                feature.ColumnValues.Add(key, columnValues[key]);
            }

            return feature;
        }
    }
}
